@@ -78,7 +78,10 @@ class Compiler:
 		return self.program.code[pc]
 
 	# Add a command to the code list
-	def add(self, command):
+	def addCommand(self, command):
+		if len(self.code) == 0:
+			if self.program.usingGraphics:
+				pass
 		self.code.append(command)
 
 	def isSymbol(self):
@@ -161,7 +164,7 @@ class Compiler:
 		command['locked'] = False
 		command['extra'] = extra
 		if 'keyword' in command: command['hasValue'] = self.hasValue(command['keyword'])
-		self.add(command)
+		self.addCommand(command)
 		return True
 
 	# Compile the current token
@@ -209,7 +212,7 @@ class Compiler:
 		self.index = index
 		while True:
 			token = self.tokens[self.index]
-			keyword = token.token
+#			keyword = token.token
 			if self.debugCompile: print(self.script.lines[token.lino])
 #			if keyword != 'else':
 			if self.compileOne() == True:
@@ -223,3 +226,6 @@ class Compiler:
 
 	def compileFromHere(self, stopOn):
 		return self.compileFrom(self.getIndex(), stopOn)
+
+	def compileFromStart(self):
+		return self.compileFrom(0, [])
\ No newline at end of file
