@@ -1,7 +1,7 @@
 import sys
 from functools import partial
 from .ec_handler import Handler
-from .ec_classes import RuntimeError
+from .ec_classes import RuntimeError, Object
 from .ec_border import Border
 from PySide6.QtCore import Qt, QTimer, Signal, QRect
 from PySide6.QtGui import QPixmap, QPainter
@@ -26,6 +26,7 @@ from PySide6.QtWidgets import (
     QSlider,
     QSpinBox,
     QTimeEdit,
+    QLayout,
     QVBoxLayout,
     QHBoxLayout,
     QGridLayout,
@@ -67,13 +68,25 @@ class Graphics(Handler):
             'combobox',
             'widget'
             ]
+    
+    def setWidget(self, record, widget):
+        if record['index'] >= record['elements']:
+            RuntimeError(self.program, f'Index out of range for widget {record["name"]}')
+        if not 'widget' in record:
+            record['widget'] = [None] * record['elements']
+        while len(record['widget']) < record['elements']:
+            record['widget'].append(None)
+        record['widget'][record['index']] = widget
+    
+    def getWidget(self, record):
+        if 'widget' in record and record['widget'] != None:
+            if record['keyword'] in ['layout', 'group']: return record['widget'] 
+            return record['widget'][record['index']]
+        else:
+            return None
 
     def dialogTypes(self):
         return ['confirm', 'lineedit', 'multiline', 'generic']
-    
-    def getWidget(self, record):
-        if record['keyword'] in ['pushbutton']: return self.getSymbolContent(record)
-        else: return record['widget']
 
     class ClickableLineEdit(QLineEdit):
         clicked = Signal()
@@ -114,7 +127,7 @@ class Graphics(Handler):
     # (2) add {widget} to {layout}
     # (3) add stretch {widget} to {layout}
     # (4) add stretch to {layout}
-    # (5) add spacer {size} to {layout}
+    # (5) add spacer [size] {size} to {layout}
     # (6) add {widget} at {col} {row} in {grid layout}
     def k_add(self, command):
         def addToLayout():
@@ -146,6 +159,7 @@ class Graphics(Handler):
         
         elif token == 'spacer':
             self.nextToken()
+            self.skip('size')
             command['widget'] = 'spacer'
             command['size'] = self.nextValue()
             self.skip('to')
@@ -186,19 +200,19 @@ class Graphics(Handler):
     def r_add(self, command):
         if 'value' in command:
             value = self.getRuntimeValue(command['value'])
-            widget = self.getVariable(command['widget'])
-            if widget['keyword'] == 'listbox':
-                widget['widget'].addItem(value)
-            elif widget['keyword'] == 'combobox':
-                if isinstance(value, list): widget['widget'].addItems(value)
-                else: widget['widget'].addItem(value)
+            record = self.getVariable(command['widget'])
+            if record['keyword'] == 'listbox':
+                self.getWidget(record).addItem(value)
+            elif record['keyword'] == 'combobox':
+                if isinstance(value, list): record['widget'].addItems(value)
+                else: self.getWidget(record).addItem(value)
         elif 'row' in command and 'col' in command:
             layout = self.getVariable(command['layout'])['widget']
-            widgetVar = self.getVariable(command['widget'])
-            widget = widgetVar['widget']
+            record = self.getVariable(command['widget'])
+            widget = self.getWidget(record)
             row = self.getRuntimeValue(command['row'])
             col = self.getRuntimeValue(command['col'])
-            if widgetVar['keyword'] == 'layout':
+            if record['keyword'] == 'layout':
                 layout.addLayout(widget, row, col)
             else:
                 layout.addWidget(widget, row, col)
@@ -272,8 +286,41 @@ class Graphics(Handler):
         return False
     
     def r_clear(self, command):
-        widget = self.getVariable(command['name'])['widget']
-        widget.clear()
+
+        def clearLayout(layout: QLayout) -> None:
+            if layout is None:
+                return
+            while layout.count() > 0:
+                item = layout.takeAt(0)
+                if item is None:
+                    continue
+                widget = item.widget()
+                if widget is not None:
+                    # Delete the widget
+                    widget.deleteLater()
+                elif item.layout() is not None:
+                    # Recursively clear sub-layout
+                    clearLayout(item.layout())
+                    item.layout().deleteLater()
+                # The QLayoutItem will be automatically cleaned up by Qt
+
+        def clearWidget(widget: QWidget) -> None:
+            if widget is None:
+                return
+            # Clear the layout first
+            layout = widget.layout()
+            if layout is not None:
+                clearLayout(layout)
+                layout.deleteLater()
+            # Clear any remaining child widgets
+            child_widgets = widget.findChildren(QWidget, "", Qt.FindDirectChildrenOnly)
+            for child in child_widgets:
+                child.deleteLater()
+
+        widget = self.getWidget(self.getVariable(command['name']))
+        clearWidget(widget)
+        return self.nextPC()
+
         return self.nextPC()
 
     # close {window}
@@ -370,17 +417,18 @@ class Graphics(Handler):
         return True
 
     def k_createPushbutton(self, command):
-        text = ''
         while True:
             token = self.peek()
             if token == 'text':
                 self.nextToken()
-                text = self.nextValue()
+                command['text'] = self.nextValue()
+            elif token == 'icon':
+                self.nextToken()
+                command['icon'] = self.nextValue()
             elif token == 'size':
                 self.nextToken()
                 command['size'] = self.nextValue()
             else: break
-        command['text'] = text
         self.add(command)
         return True
 
@@ -562,19 +610,33 @@ class Graphics(Handler):
             elif alignment == 'justify': label.setAlignment(Qt.AlignJustify)
         if 'expand' in command:
             label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
-        record['widget'] = label
+        self.setWidget(record, label)
         return self.nextPC()
     
     def r_createPushbutton(self, command, record):
-        text = self.getRuntimeValue(command['text'])
-        pushbutton = QPushButton(text)
-        pushbutton.setAccessibleName(text)
         if 'size' in command:
-            fm = pushbutton.fontMetrics()
-            c = pushbutton.contentsMargins()
-            w = fm.horizontalAdvance('m') * self.getRuntimeValue(command['size']) + c.left()+c.right()
-            pushbutton.setMaximumWidth(w)
+            size = self.getRuntimeValue(command['size'])
+        else: size = None
+        if 'icon' in command:
+            iconPath = self.getRuntimeValue(command['icon'])
+            pixmap = QPixmap(iconPath)
+            if pixmap.isNull():
+                RuntimeError(self.program, f'Icon not found: {iconPath}')
+            icon = pixmap.scaledToHeight(size if size != None else 24, Qt.SmoothTransformation)
+            pushbutton = QPushButton()
+            pushbutton.setIcon(icon)
+            pushbutton.setIconSize(icon.size())
+        elif 'text' in command:
+            text = self.getRuntimeValue(command['text'])
+            pushbutton = QPushButton(text)
+            pushbutton.setAccessibleName(text)
+            if size != None:
+                fm = pushbutton.fontMetrics()
+                c = pushbutton.contentsMargins()
+                w = fm.horizontalAdvance('m') * self.getRuntimeValue(command['size']) + c.left()+c.right()
+                pushbutton.setMaximumWidth(w)
         self.putSymbolValue(record, pushbutton)
+        self.setWidget(record, pushbutton)
         return self.nextPC()
     
     def r_createCheckBox(self, command, record):
@@ -596,7 +658,7 @@ class Graphics(Handler):
             }
         """)
         checkbox.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Preferred)
-        record['widget'] = checkbox
+        self.setWidget(record, checkbox)
         return self.nextPC()
     
     def r_createLineEdit(self, command, record):
@@ -607,7 +669,7 @@ class Graphics(Handler):
         c = lineinput.contentsMargins()
         w = fm.horizontalAdvance('x') * self.getRuntimeValue(command['size']) +m.left()+m.right()+c.left()+c.right()
         lineinput.setMaximumWidth(w)
-        record['widget'] = lineinput
+        self.setWidget(record, lineinput)
         return self.nextPC()
     
     def r_createMultiLineEdit(self, command, record):
@@ -617,19 +679,19 @@ class Graphics(Handler):
         charHeight = fontMetrics.height()
         textinput.setFixedWidth(charWidth * self.getRuntimeValue(command['cols']))
         textinput.setFixedHeight(charHeight * self.getRuntimeValue(command['rows']))
-        record['widget'] = textinput
+        self.setWidget(record, textinput)
         return self.nextPC()
     
     def r_createListWidget(self, command, record):
-        record['widget'] = QListWidget()
+        self.setWidget(record, QListWidget())
         return self.nextPC()
     
     def r_createComboBox(self, command, record):
-        record['widget'] = QComboBox()
+        self.setWidget(record, QComboBox())
         return self.nextPC()
     
     def r_createWidget(self, command, record):
-        record['widget'] = QWidget()
+        self.setWidget(record, QWidget())
         return self.nextPC()
     
     def r_createDialog(self, command, record):
@@ -757,8 +819,24 @@ class Graphics(Handler):
     def r_group(self, command):
         return self.nextPC()
 
+    # hide {widget}
+    def k_hide(self, command):
+        if self.nextIsSymbol():
+            record = self.getSymbolRecord()
+            if self.isWidget(record['keyword']):
+                command['widget'] = record['name']
+                self.add(command)
+                return True
+        return False
+        
+    def r_hide(self, command):
+        record = self.getVariable(command['widget'])
+        if 'widget' in record: self.getWidget(record).hide()
+        return self.nextPC()
+
     # Initialize the graphics environment
     def k_init(self, command):
+        # return True
         if self.nextIs('graphics'):
             self.add(command)
             return True
@@ -770,7 +848,22 @@ class Graphics(Handler):
         self.program.screenWidth = screen[0]
         self.program.screenHeight = screen[1]
         print(f'Screen: {self.program.screenWidth}x{self.program.screenHeight}')
-        return self.nextPC()
+        # return self.nextPC()
+        def on_last_window_closed():
+            self.program.kill()
+        def init():
+            self.program.flush(self.nextPC())
+        def flush():
+            if not self.blocked:
+                if self.runOnTick != 0:
+                    self.program.run(self.runOnTick)
+                self.program.flushCB()
+        timer = QTimer()
+        timer.timeout.connect(flush)
+        timer.start(10)
+        QTimer.singleShot(500, init)
+        self.app.lastWindowClosed.connect(on_last_window_closed)
+        self.app.exec()
 
     # Declare a label variable
     def k_label(self, command):
@@ -887,7 +980,7 @@ class Graphics(Handler):
     
     def r_on(self, command):
         def run(widget, record):
-            for i, w in enumerate(record['value']):
+            for i, w in enumerate(record['widget']):
                 if w == widget:
                     record['index'] = i
                     self.run(command['goto'])
@@ -941,10 +1034,10 @@ class Graphics(Handler):
         record = self.getVariable(command['name'])
         if variant == 'current':
             if record['keyword'] == 'combobox':
-                widget = record['widget']
+                widget = self.getWidget(record)
                 widget.removeItem(widget.currentIndex())
             if record['keyword'] == 'listbox':
-                widget = record['widget']
+                widget = self.getWidget(record)
                 selectedItem = widget.currentItem()
                 if selectedItem:
                     row = widget.row(selectedItem)
@@ -969,7 +1062,7 @@ class Graphics(Handler):
         return False
     
     def r_select(self, command):
-        widget = self.getVariable(command['widget'])['widget']
+        widget = self.getWidget(self.getVariable(command['widget']))
         if 'index' in command:
             index = self.getRuntimeValue(command['index'])
         else:
@@ -1012,9 +1105,10 @@ class Graphics(Handler):
                     self.skip('to')
                     if self.nextIsSymbol():
                         record = self.getSymbolRecord()
-                        command['layout'] = record['name']
-                        self.add(command)
-                        return True
+                        if record['keyword'] == 'layout':
+                            command['layout'] = record['name']
+                            self.add(command)
+                            return True
         elif token == 'spacing':
             self.skip('of')
             if self.nextIsSymbol():
@@ -1122,25 +1216,26 @@ class Graphics(Handler):
     def r_set(self, command):
         what = command['what']
         if what == 'height':
-            widget = self.getVariable(command['name'])['widget']
+            widget = self.getWidget(self.getVariable(command['name']))
             widget.setFixedHeight(self.getRuntimeValue(command['value']))
         elif what == 'width':
-            widget = self.getVariable(command['name'])['widget']
+            widget = self.getWidget(self.getVariable(command['name']))
             widget.setFixedWidth(self.getRuntimeValue(command['value']))
         elif what == 'layout':
-            content = self.getVariable(command['layout'])['widget']
+            record = self.getVariable(command['layout'])
+            layout = record['widget']
             record = self.getVariable(command['name'])
             keyword = record['keyword']
             if keyword == 'window':
                 window = record['window']
                 container = QWidget()
-                container.setLayout(content)
+                container.setLayout(layout)
                 window.setCentralWidget(container)
             elif keyword == 'widget':
-                widget = record['widget']
-                widget.setLayout(content)
+                widget = self.getWidget(record)
+                widget.setLayout(layout)
         elif what == 'spacing':
-            layout = self.getVariable(command['name'])['widget']
+            layout = self.getWidget(self.getVariable(command['name']))
             layout.setSpacing(self.getRuntimeValue(command['value']))
         elif what == 'text':
             record = self.getVariable(command['name'])
@@ -1158,7 +1253,7 @@ class Graphics(Handler):
             record = self.getVariable(command['name'])
             if record['keyword'] == 'checkbox':
                 state = self.getRuntimeValue(command['value'])
-                record['widget'].setChecked(state)
+                self.getWidget(record).setChecked(state)
         elif what == 'alignment':
             widget = self.getVariable(command['name'])['widget']
             flags = command['value']
@@ -1197,6 +1292,7 @@ class Graphics(Handler):
 
     # show {window}
     # show {dialog}
+    # show {widget}
     # show {messagebox} giving {result}}
     def k_show(self, command):
         if self.nextIsSymbol():
@@ -1210,6 +1306,10 @@ class Graphics(Handler):
                 command['dialog'] = record['name']
                 self.add(command)
                 return True
+            elif self.isWidget(keyword):
+                command['name'] = record['name']
+                self.add(command)
+                return True
             elif keyword == 'messagebox':
                 command['messagebox'] = record['name']
                 self.skip('giving')
@@ -1270,6 +1370,9 @@ class Graphics(Handler):
                 if dialog.exec() == QDialog.Accepted:
                     record['result'] = dialog.textEdit.toPlainText()
                 else: record['result'] = dialog.value
+        elif 'name' in command:
+            record = self.getVariable(command['name'])
+            if 'widget' in record: self.getWidget(record).show()
         return self.nextPC()
 
     # Start the graphics
@@ -1280,21 +1383,22 @@ class Graphics(Handler):
         return False
         
     def r_start(self, command):
-        def on_last_window_closed():
-            self.program.kill()
-        def init():
-            self.program.flush(self.nextPC())
-        def flush():
-            if not self.blocked:
-                if self.runOnTick != 0:
-                    self.program.run(self.runOnTick)
-                self.program.flushCB()
-        timer = QTimer()
-        timer.timeout.connect(flush)
-        timer.start(10)
-        QTimer.singleShot(500, init)
-        self.app.lastWindowClosed.connect(on_last_window_closed)
-        self.app.exec()
+        return self.nextPC()
+        # def on_last_window_closed():
+        #     self.program.kill()
+        # def init():
+        #     self.program.flush(self.nextPC())
+        # def flush():
+        #     if not self.blocked:
+        #         if self.runOnTick != 0:
+        #             self.program.run(self.runOnTick)
+        #         self.program.flushCB()
+        # timer = QTimer()
+        # timer.timeout.connect(flush)
+        # timer.start(10)
+        # QTimer.singleShot(500, init)
+        # self.app.lastWindowClosed.connect(on_last_window_closed)
+        # self.app.exec()
 
     # Declare a widget variable
     def k_widget(self, command):
@@ -1364,38 +1468,38 @@ class Graphics(Handler):
         symbolRecord = self.getVariable(symbolRecord['name'])
         keyword = symbolRecord['keyword']
         if keyword == 'pushbutton':
-            pushbutton = self.getSymbolContent(symbolRecord) # symbolRecord['widget']
+            pushbutton = self.getWidget(symbolRecord)
             v = {}
             v['type'] = 'text'
             v['content'] = pushbutton.accessibleName()
             return v
         elif keyword == 'lineinput':
-            lineinput = symbolRecord['widget']
+            lineinput = self.getWidget(symbolRecord)
             v = {}
             v['type'] = 'text'
             v['content'] = lineinput.displayText()
             return v
         elif keyword == 'multiline':
-            multiline = symbolRecord['widget']
+            multiline = self.getWidget(symbolRecord)
             v = {}
             v['type'] = 'text'
             v['content'] = multiline.toPlainText()
             return v
         elif keyword == 'combobox':
-            combobox = symbolRecord['widget']
+            combobox = self.getWidget(symbolRecord)
             v = {}
             v['type'] = 'text'
             v['content'] = combobox.currentText()
             return v
         elif keyword == 'listbox':
-            listbox = symbolRecord['widget']
+            listbox = self.getWidget(symbolRecord)
             content = listbox.currentItem().text()
             v = {}
             v['type'] = 'text'
             v['content'] = content
             return v
         elif keyword == 'checkbox':
-            checkbox = symbolRecord['widget']
+            checkbox =self.getWidget(symbolRecord)
             content = checkbox.isChecked()
             v = {}
             v['type'] = 'boolean'
@@ -1412,7 +1516,7 @@ class Graphics(Handler):
     def v_count(self, v):
         record = self.getVariable(v['name'])
         keyword = record['keyword']
-        widget = record['widget']
+        widget = self.getWidget(record)
         if keyword in ['combobox', 'listbox']: content = widget.count()
         value = {}
         value['type'] = 'int'
@@ -1422,7 +1526,7 @@ class Graphics(Handler):
     def v_current(self, v):
         record = self.getVariable(v['name'])
         keyword = record['keyword']
-        widget = record['widget']
+        widget = self.getWidget(record)
         if keyword == 'listbox': content = widget.currentItem().text()
         value = {}
         value['type'] = 'text'
@@ -1432,8 +1536,9 @@ class Graphics(Handler):
     #############################################################################
     # Compile a condition
     def compileCondition(self):
-        condition = {}
-        return condition
+        condition = Object()
+        condition.negate = False
+        return None
 
     #############################################################################
     # Condition handlers
@@ -1442,4 +1547,4 @@ class Graphics(Handler):
     # Force the application to exit
     def force_exit(self):
         QApplication.quit()  # Gracefully close the application
-        sys.exit(0)          # Force a complete system exit
+        sys.exit(0)          # Force a complete system exit
\ No newline at end of file
